"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CypherAccount = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("../utils");
const mango_client_1 = require("@blockworks-foundation/mango-client");
const pda_1 = require("../utils/pda");
const ordersAccount_1 = require("./ordersAccount");
const associated_token_1 = require("@project-serum/associated-token");
class CypherAccount {
    constructor(client, address, state, _onStateUpdate, _errorCallback) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        this._errorCallback = _errorCallback;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, clearing, authority, accountNumber = 0) {
        const [account, bump] = (0, utils_1.deriveAccountAddress)(authority, accountNumber, client.cypherPID);
        const ix = await client.methods
            .createAccount(accountNumber, bump)
            .accountsStrict({
            clearing,
            masterAccount: account,
            authority,
            payer: client.walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .instruction();
        return {
            account,
            ixs: [ix],
            signers: []
        };
    }
    static async createWhitelisted(client, clearing, whitelist, authority, accountNumber = 0) {
        const [account, bump] = (0, utils_1.deriveAccountAddress)(authority, accountNumber, client.cypherPID);
        const ix = await client.methods
            .createWhitelistedAccount(accountNumber, bump)
            .accountsStrict({
            clearing,
            whitelist,
            masterAccount: account,
            authority,
            payer: client.walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .instruction();
        return {
            account,
            ixs: [ix],
            signers: []
        };
    }
    static async load(client, address, onStateUpdateHandler, errorCallback) {
        const state = await client.accounts.cypherAccount.fetchNullable(address);
        if (state === null)
            return null;
        return new CypherAccount(client, address, state, onStateUpdateHandler, errorCallback);
    }
    getSubAccounts() {
        const subAccounts = [];
        for (const subAccountCache of this.state.subAccountCaches) {
            if (!subAccountCache.subAccount.equals(web3_js_1.PublicKey.default)) {
                subAccounts.push(subAccountCache.subAccount);
            }
        }
        return subAccounts;
    }
    getCRatio(cacheAccount, subAccounts) {
        const totalAssetsValue = mango_client_1.I80F48.fromNumber(0);
        const totalAssetsValueUnweighted = mango_client_1.I80F48.fromNumber(0);
        const totalVolatileAssetsValue = mango_client_1.I80F48.fromNumber(0);
        const totalLiabilitiesValue = mango_client_1.I80F48.fromNumber(0);
        const totalLiabilitiesValueUnweighted = mango_client_1.I80F48.fromNumber(0);
        const totalVolatileLiabilitiesValue = mango_client_1.I80F48.fromNumber(0);
        for (const subAccount of subAccounts) {
            const { assetsValue, assetsValueUnweighted, volatileAssetsValue } = subAccount.getAssetsValue(cacheAccount);
            const { liabilitiesValue, liabilitiesValueUnweighted, volatileLiabilitiesValue } = subAccount.getLiabilitiesValue(cacheAccount);
            if (subAccount.state.marginingType.cross) {
                totalAssetsValue.iadd(assetsValue);
                totalAssetsValueUnweighted.iadd(assetsValueUnweighted);
                totalVolatileAssetsValue.iadd(volatileAssetsValue);
                totalLiabilitiesValue.iadd(liabilitiesValue);
                totalLiabilitiesValueUnweighted.iadd(liabilitiesValueUnweighted);
                totalVolatileLiabilitiesValue.iadd(volatileLiabilitiesValue);
            }
            // const cRatio = assetsValue.div(liabsValue);
            // console.log(
            //   'SubAccount:',
            //   subAccount.address.toString(),
            //   'Assets:',
            //   assetsValue.toFixed(4),
            //   'Liabilities:',
            //   liabsValue.toFixed(4),
            //   'C-Ratio:',
            //   cRatio.toFixed(4)
            // );
        }
        return {
            assetsValue: totalAssetsValue,
            assetsValueUnweighted: totalAssetsValueUnweighted,
            volatileAssetsValue: totalVolatileAssetsValue,
            liabilitiesValue: totalLiabilitiesValue,
            liabilitiesValueUnweighted: totalLiabilitiesValueUnweighted,
            volatileLiabilitiesValue: totalVolatileLiabilitiesValue,
            cRatio: totalLiabilitiesValue.isZero()
                ? new mango_client_1.I80F48(mango_client_1.I80F48.MAX_BN)
                : totalAssetsValue.div(totalLiabilitiesValue),
            volatileCRatio: totalVolatileLiabilitiesValue.isZero()
                ? new mango_client_1.I80F48(mango_client_1.I80F48.MAX_BN)
                : totalVolatileAssetsValue.div(totalVolatileLiabilitiesValue)
        };
    }
    subscribe() {
        this.removeListener();
        try {
            this.addListener();
        }
        catch (error) {
            if (this._errorCallback) {
                this._errorCallback(error);
            }
        }
    }
    addListener() {
        this._listener = this.client.connection.onAccountChange(this.address, ({ data }) => {
            this.state = this.client.program.coder.accounts.decode('CypherAccount', data);
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        }, 'processed');
    }
    removeListener() {
        if (this._listener)
            this.client.connection.removeAccountChangeListener(this._listener);
    }
    async unsubscribe() {
        this.removeListener();
    }
    getSpotOpenOrdersAddress(dexMarket, subAccount) {
        const [address] = (0, pda_1.deriveSerumOrdersAccountAddress)(dexMarket, this.address, subAccount, this.client.cypherPID);
        return address;
    }
    getDerivativesOpenOrdersAddress(market) {
        const [address] = (0, pda_1.deriveOrdersAccountAddress)(this.address, market, this.client.cypherPID);
        return address;
    }
    async hasSpotOpenOrders(dexMarket, subAccount) {
        const spotOpenOrders = this.getSpotOpenOrdersAddress(dexMarket, subAccount);
        const accountState = await this.client.connection.getAccountInfo(spotOpenOrders);
        if (accountState) {
            return true;
        }
        else {
            return false;
        }
    }
    async hasDerivativeOpenOrders(market) {
        const ordersAccountAddress = this.getDerivativesOpenOrdersAddress(market);
        const accountState = await this.client.accounts.ordersAccount.fetchNullable(ordersAccountAddress);
        if (accountState) {
            return true;
        }
        else {
            return false;
        }
    }
    async getSpotMarketOrders(pool) {
        if (pool.market)
            return await pool.market.loadOrdersForOwner(this.client.connection, this.address);
        return [];
    }
    async getDerivativeMarketOrders(market) {
        const ordersAccountAddress = this.getDerivativesOpenOrdersAddress(market.address);
        const ordersAccount = await ordersAccount_1.DerivativesOrdersAccount.load(this.client, ordersAccountAddress);
        return ordersAccount.getOrders(market);
    }
    async getAssociatedTokenAddress(tokenMint) {
        return (0, associated_token_1.getAssociatedTokenAddress)(this.client.walletPubkey, tokenMint);
    }
}
exports.CypherAccount = CypherAccount;
//# sourceMappingURL=account.js.map