"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CacheAccount = void 0;
const web3_js_1 = require("@solana/web3.js");
class CacheAccount {
    constructor(client, address, state, _onStateUpdate, _errorCallback) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        this._errorCallback = _errorCallback;
        this._cacheListeners = new Map();
        _onStateUpdate && this.subscribe();
    }
    static async create(client, clearing, authority) {
        const cacheAccountFull = web3_js_1.Keypair.generate();
        const preIx = await client.accounts.cacheAccount.createInstruction(cacheAccountFull, 65576);
        const ix = await client.methods
            .initCacheAccount()
            .accountsStrict({
            clearing,
            cacheAccount: cacheAccountFull.publicKey,
            authority
        })
            .instruction();
        return {
            cacheAccount: cacheAccountFull.publicKey,
            ixs: [preIx, ix],
            signers: [cacheAccountFull]
        };
    }
    get connection() {
        return this.client.connection;
    }
    static async load(client, address, onStateUpdateHandler, errorCallback) {
        const state = (await client.accounts.cacheAccount.fetchNullable(address));
        return new CacheAccount(client, address, state, onStateUpdateHandler, errorCallback);
    }
    getCaches() {
        const caches = [];
        for (const cache of this.state.caches) {
            if (!cache.oracleProducts.equals(web3_js_1.PublicKey.default)) {
                caches.push(cache);
            }
        }
        return caches;
    }
    getCache(idx) {
        return this.state.caches[idx];
    }
    addCacheListener(callback, idx, errorCallback) {
        this.removeCacheListener(idx);
        this._cacheListeners.set(idx, callback);
        try {
            this.subscribe();
        }
        catch (error) {
            errorCallback(error);
        }
    }
    removeCacheListener(idx) {
        if (this._cacheListeners.get(idx))
            this._cacheListeners.delete(idx);
    }
    subscribe() {
        this.removeListener();
        try {
            this.addListener();
        }
        catch (error) {
            if (this._errorCallback) {
                this._errorCallback(error);
            }
        }
    }
    addListener() {
        this._listener = this.client.connection.onAccountChange(this.address, ({ data }) => {
            this.state = this.client.program.coder.accounts.decode('CacheAccount', data);
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
            for (const [key, value] of this._cacheListeners) {
                const cache = this.getCache(key);
                value(cache);
            }
        }, 'processed');
    }
    removeListener() {
        if (this._listener)
            this.client.connection.removeAccountChangeListener(this._listener);
    }
    async unsubscribe() {
        this.removeListener();
    }
}
exports.CacheAccount = CacheAccount;
//# sourceMappingURL=cacheAccount.js.map