"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FuturesMarket = void 0;
const web3_js_1 = require("@solana/web3.js");
const instructions_1 = require("../instructions");
const utils_1 = require("../utils");
class FuturesMarket {
    constructor(client, address, state, _onStateUpdate, _errorCallback) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        this._errorCallback = _errorCallback;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, authority, clearing, cacheAccount, quotePool, oracleProducts, args) {
        const [market, marketBump] = (0, utils_1.deriveMarketAddress)(args.marketName, client.cypherPID);
        args.marketBump = marketBump;
        const { orderbook, eventQueue, bids, asks, priceHistory, ixs, signers } = await (0, instructions_1.makeCreateMarketAccountsIxs)(client);
        const tx = new web3_js_1.Transaction();
        tx.add(...ixs);
        await client.sendAndConfirm(tx, signers);
        await client.methods
            .createFuturesMarket({
            ...args,
            marketType: args.marketType,
            deliveryType: args.deliveryType
        })
            .accountsStrict({
            clearing,
            cacheAccount,
            market,
            priceHistory,
            oracleProducts,
            quotePool: quotePool.address,
            orderbook,
            bids,
            asks,
            eventQueue,
            authority,
            payer: client.walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId,
            rent: web3_js_1.SYSVAR_RENT_PUBKEY
        })
            .rpc();
        return market;
    }
    static async load(client, address, onStateUpdateHandler, errorCallback) {
        const state = (await client.accounts.futuresMarket.fetchNullable(address));
        return new FuturesMarket(client, address, state, onStateUpdateHandler, errorCallback);
    }
    static async loadAll(client) {
        const queryResult = await client.accounts.futuresMarket.all();
        return queryResult.map((result) => new FuturesMarket(client, result.publicKey, result.account));
    }
    get marketPrice() {
        return this.state.marketPrice;
    }
    get positionsCount() {
        return this.state.positionsCount;
    }
    get totalBorrows() {
        return this.state.totalBorrows;
    }
    get totalPurchased() {
        return this.state.totalPurchased;
    }
    get totalRaised() {
        return this.state.totalRaised;
    }
    get tokenSupply() {
        return this.state.tokenSupply;
    }
    get marketActivatesAt() {
        return (0, utils_1.bnToDate)(this.state.activatesAt);
    }
    get marketExpiresAt() {
        return (0, utils_1.bnToDate)(this.state.expiresAt);
    }
    subscribe() {
        this.removeListener();
        try {
            this.addListener();
        }
        catch (error) {
            if (this._errorCallback) {
                this._errorCallback(error);
            }
        }
    }
    addListener() {
        this._listener = this.client.connection.onAccountChange(this.address, ({ data }) => {
            this.state = this.client.program.coder.accounts.decode('FuturesMarket', data);
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        }, 'processed');
    }
    removeListener() {
        if (this._listener)
            this.client.connection.removeAccountChangeListener(this._listener);
    }
    async unsubscribe() {
        this.removeListener();
    }
}
exports.FuturesMarket = FuturesMarket;
//# sourceMappingURL=futuresMarket.js.map