"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CypherSubAccount = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("../utils");
const mango_client_1 = require("@blockworks-foundation/mango-client");
const spotPosition_1 = require("../viewers/spotPosition");
const derivativePosition_1 = require("../viewers/derivativePosition");
const shared_1 = require("../constants/shared");
class CypherSubAccount {
    constructor(client, address, state, _onStateUpdate, _errorCallback) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        this._errorCallback = _errorCallback;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, authority, masterAccount, accountNumber = 0, accountAlias = Array(32).fill(0)) {
        const [subAccount, bump] = (0, utils_1.deriveSubAccountAddress)(masterAccount, accountNumber, client.cypherPID);
        const ix = await client.methods
            .createSubAccount(accountNumber, bump, accountAlias)
            .accountsStrict({
            masterAccount,
            subAccount,
            authority,
            payer: client.walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .instruction();
        return {
            subAccount,
            ixs: [ix],
            signers: []
        };
    }
    static async load(client, address, onStateUpdateHandler, errorCallback) {
        const state = (await client.accounts.cypherSubAccount.fetchNullable(address));
        return new CypherSubAccount(client, address, state, onStateUpdateHandler, errorCallback);
    }
    getSpotPosition(tokenMint) {
        for (const position of this.state.positions) {
            if (position.spot.tokenMint.equals(tokenMint)) {
                return new spotPosition_1.SpotPosition(position.spot);
            }
        }
        return null;
    }
    // The position does not account for deposit or borrow interest
    // To get the actual value of the position
    // If its a deposit, multiply it by the deposit interest rate
    // If its a borrow, multiply it by the borrow interest rate
    // The index can be found in the resp cypher pool
    getSpotPositionsInfo() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                positions.push(position.spot);
            }
        }
        return positions;
    }
    getSpotPositions() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                positions.push(new spotPosition_1.SpotPosition(position.spot));
            }
        }
        return positions;
    }
    getDerivativePosition(market) {
        for (const position of this.state.positions) {
            if (position.derivative.market.equals(market)) {
                return new derivativePosition_1.DerivativePosition(position.derivative);
            }
        }
        return null;
    }
    getDerivativePositionsInfo() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.derivative.market.equals(web3_js_1.PublicKey.default)) {
                positions.push(position.derivative);
            }
        }
        return positions;
    }
    getDerivativePositions() {
        const positions = [];
        for (const position of this.state.positions) {
            if (!position.derivative.market.equals(web3_js_1.PublicKey.default)) {
                positions.push(new derivativePosition_1.DerivativePosition(position.derivative));
            }
        }
        return positions;
    }
    getAssetsValue(cacheAccount) {
        const assetsValueUnweighted = mango_client_1.I80F48.fromNumber(0);
        const assetsValue = mango_client_1.I80F48.fromNumber(0);
        const volatileAssetsValue = mango_client_1.I80F48.fromNumber(0);
        const cumPcTotal = mango_client_1.I80F48.fromNumber(0);
        for (const { spot, derivative } of this.state.positions) {
            if (!spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(spot.cacheIndex);
                const decimals = priceCache.decimals;
                const openOrdersCache = spot.openOrdersCache;
                const oraclePrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                const weight = mango_client_1.I80F48.fromNumber(priceCache.spotInitAssetWeight).div(mango_client_1.I80F48.fromNumber(100));
                const basePosition = new mango_client_1.I80F48(spot.position);
                if (basePosition.isPos()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition, priceCache.decimals).mul(oraclePrice);
                    assetsValueUnweighted.iadd(positionValue);
                    assetsValue.iadd(positionValue.mul(weight));
                    if (priceCache.safeguard) {
                        volatileAssetsValue.iadd(positionValue.mul(weight));
                    }
                    // console.log(
                    //   'Asset ----- Token: ',
                    //   spot.tokenMint.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, priceCache.decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   oraclePrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
                if (!openOrdersCache.coinTotal.eq(mango_client_1.ZERO_BN)) {
                    const coinTotalIncl = (0, utils_1.splToUiAmountFixed)(mango_client_1.I80F48.fromU64(openOrdersCache.coinTotal), decimals).mul(oraclePrice);
                    assetsValueUnweighted.iadd(coinTotalIncl);
                    assetsValue.iadd(coinTotalIncl.mul(weight));
                    if (priceCache.safeguard) {
                        volatileAssetsValue.iadd(coinTotalIncl.mul(weight));
                    }
                }
                const pcTotalIncl = (0, utils_1.splToUiAmountFixed)(mango_client_1.I80F48.fromU64(openOrdersCache.pcTotal), shared_1.QUOTE_TOKEN_DECIMALS);
                cumPcTotal.iadd(pcTotalIncl);
            }
            if (!derivative.market.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(derivative.cacheIndex);
                const openOrdersCache = derivative.openOrdersCache;
                let decimals = 0;
                if (derivative.marketType.perpetualFuture) {
                    decimals = priceCache.perpDecimals;
                }
                else {
                    decimals = priceCache.futuresDecimals;
                }
                let derivPrice = mango_client_1.ZERO_I80F48;
                if (derivative.marketType.perpetualFuture) {
                    derivPrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                }
                else {
                    derivPrice = new mango_client_1.I80F48(priceCache.marketPrice);
                }
                let weight = mango_client_1.ZERO_I80F48;
                if (derivative.marketType.perpetualFuture) {
                    weight = mango_client_1.I80F48.fromNumber(priceCache.perpInitAssetWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                else {
                    weight = mango_client_1.I80F48.fromNumber(priceCache.futuresInitAssetWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                const basePosition = new mango_client_1.I80F48(derivative.basePosition);
                if (basePosition.isPos()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition, decimals).mul(derivPrice);
                    assetsValueUnweighted.iadd(positionValue);
                    assetsValue.iadd(positionValue.mul(weight));
                    if (priceCache.safeguard) {
                        volatileAssetsValue.iadd(positionValue.mul(weight));
                    }
                    // console.log(
                    //   'Asset ----- Market: ',
                    //   derivative.market.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   derivPrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
                const coinLocked = openOrdersCache.coinTotal.sub(openOrdersCache.coinFree);
                if (!coinLocked.eq(mango_client_1.ZERO_BN)) {
                    const coinTotalIncl = (0, utils_1.splToUiAmountFixed)(mango_client_1.I80F48.fromU64(coinLocked), decimals).mul(derivPrice);
                    assetsValueUnweighted.iadd(coinTotalIncl);
                    assetsValue.iadd(coinTotalIncl.mul(weight));
                    if (priceCache.safeguard) {
                        volatileAssetsValue.iadd(coinTotalIncl.mul(weight));
                    }
                }
                const pcTotalIncl = (0, utils_1.splToUiAmountFixed)(mango_client_1.I80F48.fromU64(openOrdersCache.pcTotal), shared_1.QUOTE_TOKEN_DECIMALS);
                cumPcTotal.iadd(pcTotalIncl);
            }
        }
        // this is a bit hacky because we know that the first cache is the usdc one
        // but this should ideally be changed
        const quotePriceCache = cacheAccount.getCache(0);
        const quotePositionValue = cumPcTotal.mul(new mango_client_1.I80F48(quotePriceCache.oraclePrice));
        const weight = mango_client_1.I80F48.fromNumber(quotePriceCache.spotInitAssetWeight).div(mango_client_1.I80F48.fromNumber(100));
        assetsValueUnweighted.iadd(quotePositionValue);
        assetsValue.iadd(quotePositionValue.mul(weight));
        if (quotePriceCache.safeguard) {
            volatileAssetsValue.iadd(quotePositionValue.mul(weight));
        }
        return { assetsValueUnweighted, assetsValue, volatileAssetsValue };
    }
    getLiabilitiesValue(cacheAccount) {
        const liabilitiesValueUnweighted = mango_client_1.I80F48.fromNumber(0);
        const liabilitiesValue = mango_client_1.I80F48.fromNumber(0);
        const volatileLiabilitiesValue = mango_client_1.I80F48.fromNumber(0);
        for (const { spot, derivative } of this.state.positions) {
            if (!spot.tokenMint.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(spot.cacheIndex);
                const oraclePrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                const weight = mango_client_1.I80F48.fromNumber(priceCache.spotInitLiabWeight).div(mango_client_1.I80F48.fromNumber(100));
                const basePosition = new mango_client_1.I80F48(spot.position);
                if (basePosition.isNeg()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition.abs(), priceCache.decimals).mul(oraclePrice);
                    liabilitiesValueUnweighted.iadd(positionValue);
                    liabilitiesValue.iadd(positionValue.mul(weight));
                    if (priceCache.safeguard) {
                        volatileLiabilitiesValue.iadd(positionValue.mul(weight));
                    }
                    // console.log(
                    //   'Liability ----- Token: ',
                    //   spot.tokenMint.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, priceCache.decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   oraclePrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
            }
            if (!derivative.market.equals(web3_js_1.PublicKey.default)) {
                const priceCache = cacheAccount.getCache(derivative.cacheIndex);
                let decimals = 0;
                let derivPrice = mango_client_1.ZERO_I80F48;
                let weight = mango_client_1.ZERO_I80F48;
                if (derivative.marketType.perpetualFuture) {
                    decimals = priceCache.perpDecimals;
                    derivPrice = new mango_client_1.I80F48(priceCache.oraclePrice);
                    weight = mango_client_1.I80F48.fromNumber(priceCache.perpInitLiabWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                else {
                    decimals = priceCache.futuresDecimals;
                    derivPrice = new mango_client_1.I80F48(priceCache.marketPrice);
                    weight = mango_client_1.I80F48.fromNumber(priceCache.futuresInitLiabWeight).div(mango_client_1.I80F48.fromNumber(100));
                }
                const basePosition = new mango_client_1.I80F48(derivative.basePosition);
                if (basePosition.isNeg()) {
                    const positionValue = (0, utils_1.splToUiAmountFixed)(basePosition.abs(), decimals).mul(derivPrice);
                    liabilitiesValueUnweighted.iadd(positionValue);
                    liabilitiesValue.iadd(positionValue.mul(weight));
                    if (priceCache.safeguard) {
                        volatileLiabilitiesValue.iadd(positionValue.mul(weight));
                    }
                    // console.log(
                    //   'Liability ----- Market: ',
                    //   derivative.market.toString(),
                    //   'Position (Native): ',
                    //   basePosition.toFixed(4),
                    //   'Position (UI): ',
                    //   splToUiAmountFixed(basePosition, decimals).toFixed(4),
                    //   'Weight: ',
                    //   weight.toFixed(4),
                    //   'Oracle Price: ',
                    //   derivPrice.toFixed(4),
                    //   'Position Value: ',
                    //   positionValue.toFixed(4)
                    // );
                }
            }
        }
        return {
            liabilitiesValueUnweighted,
            liabilitiesValue,
            volatileLiabilitiesValue
        };
    }
    subscribe() {
        this.removeListener();
        try {
            this.addListener();
        }
        catch (error) {
            if (this._errorCallback) {
                this._errorCallback(error);
            }
        }
    }
    addListener() {
        this._listener = this.client.connection.onAccountChange(this.address, ({ data }) => {
            this.state = this.client.program.coder.accounts.decode('CypherSubAccount', data);
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        }, 'processed');
    }
    removeListener() {
        if (this._listener)
            this.client.connection.removeAccountChangeListener(this._listener);
    }
    async unsubscribe() {
        this.removeListener();
    }
}
exports.CypherSubAccount = CypherSubAccount;
//# sourceMappingURL=subAccount.js.map