"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FuturesMarketViewer = void 0;
const aaob_1 = require("@chugach-foundation/aaob");
const web3_js_1 = require("@solana/web3.js");
const shared_1 = require("../constants/shared");
const tokenAmount_1 = require("../utils/tokenAmount");
class FuturesMarketViewer {
    constructor(client, market) {
        this.client = client;
        this.market = market;
    }
    get connection() {
        return this.client.connection;
    }
    get address() {
        return this.market.address;
    }
    /**
     * Parses the given buffer into the Slab structure
     * @param data the account info data buffer
     * @param increasing whether the data should be returned in increasing order
     * @param orderbookDepth number of levels of slab data to process
     */
    orderbookParser(data, increasing, orderbookDepth = 250) {
        const slab = aaob_1.Slab.deserialize(data, shared_1.CALLBACK_INFO_LEN);
        return slab.getL2DepthJS(orderbookDepth, increasing).map((level) => {
            const price = (0, tokenAmount_1.priceLotsToNative)(level.price.ushrn(32), this.market.state.inner.baseMultiplier, this.market.state.inner.quoteMultiplier, this.market.state.inner.config.decimals);
            const baseQty = (0, tokenAmount_1.sizeLotsToNative)(level.size, this.market.state.inner.baseMultiplier);
            return [
                (0, tokenAmount_1.splToUiAmount)(price, shared_1.QUOTE_TOKEN_DECIMALS),
                (0, tokenAmount_1.splToUiAmount)(baseQty, this.market.state.inner.config.decimals)
            ];
        });
    }
    eventQueueParser(data) {
        return aaob_1.EventQueue.parse(shared_1.CALLBACK_INFO_LEN, data);
    }
    fillsParser(data) {
        const eq = this.eventQueueParser(data);
        const events = [...Array(eq.header.count.toNumber()).keys()]
            .map((e) => eq.peekAt(e))
            .filter((e) => e instanceof aaob_1.EventFill);
        return events.map((fill) => {
            const side = (0, tokenAmount_1.getSideFromKey)(fill.makerOrderId);
            return {
                side: side,
                price: (0, tokenAmount_1.splToUiAmount)((0, tokenAmount_1.priceLotsToNative)((0, aaob_1.getPriceFromKey)(fill.makerOrderId).ushrn(32), this.market.state.inner.baseMultiplier, this.market.state.inner.quoteMultiplier, this.market.state.inner.config.decimals), shared_1.QUOTE_TOKEN_DECIMALS),
                amount: (0, tokenAmount_1.splToUiAmount)((0, tokenAmount_1.sizeLotsToNative)(fill.baseSize, this.market.state.inner.baseMultiplier), this.market.state.inner.config.decimals),
                makerAccount: new web3_js_1.PublicKey(Buffer.from(fill.makerCallbackInfo.slice(0, 32))),
                makerOrderId: fill.makerOrderId,
                takerAccount: new web3_js_1.PublicKey(Buffer.from(fill.takerCallbackInfo.slice(0, 32)))
            };
        });
    }
    addBidsListener(callback, errorCallback = () => { }, orderbookDepth = 250) {
        this.removeBidsListener();
        try {
            const bidsAddress = this.market.state.inner.bids;
            this._bidsListener = this.connection.onAccountChange(bidsAddress, ({ data }) => callback(this.orderbookParser(data, false, orderbookDepth)), 'processed');
        }
        catch (error) {
            errorCallback(error);
        }
    }
    removeBidsListener() {
        if (this._bidsListener)
            this.connection.removeAccountChangeListener(this._bidsListener);
    }
    addAsksListener(callback, errorCallback = () => { }, orderbookDepth = 250) {
        this.removeAsksListener();
        try {
            const asksAddress = this.market.state.inner.asks;
            this._asksListener = this.connection.onAccountChange(asksAddress, ({ data }) => callback(this.orderbookParser(data, true, orderbookDepth)), 'processed');
        }
        catch (error) {
            errorCallback(error);
        }
    }
    removeAsksListener() {
        if (this._asksListener)
            this.connection.removeAccountChangeListener(this._asksListener);
    }
    addEventQueueListener(callback, errorCallback) {
        this.removeEventQueueListener();
        try {
            this._eventQueueListener = this.connection.onAccountChange(this.market.state.inner.eventQueue, ({ data }) => callback(this.eventQueueParser(data)), 'processed');
        }
        catch (error) {
            errorCallback(error);
        }
    }
    removeEventQueueListener() {
        if (this._eventQueueListener)
            this.connection.removeAccountChangeListener(this._eventQueueListener);
    }
    addFillsListener(callback, errorCallback) {
        this.removeEventQueueListener();
        try {
            this._fillsListener = this.connection.onAccountChange(this.market.state.inner.eventQueue, ({ data }) => callback(this.fillsParser(data)), 'processed');
        }
        catch (error) {
            errorCallback(error);
        }
    }
    removeFillsListener() {
        if (this._fillsListener)
            this.connection.removeAccountChangeListener(this._fillsListener);
    }
    calcMarketOrderPrice(size, side, bids, asks) {
        const orderbook = side === 'buy' ? asks : bids;
        let acc = 0;
        let selectedOrder;
        for (const order of orderbook) {
            acc += order[1];
            if (acc >= size) {
                selectedOrder = order;
                break;
            }
        }
        if (!selectedOrder) {
            throw new Error();
        }
        if (side === 'buy')
            return selectedOrder[0] * 1.05;
        else
            return selectedOrder[0] * 0.95;
    }
    calcMarketOrderPriceForAvailableAmount(size, side, bids, asks) {
        if (!size)
            throw new Error('Size is empty');
        const orderbook = side === 'buy' ? asks : bids;
        if (orderbook.length === 0)
            return [0, 0];
        let acc = 0;
        let selectedOrder;
        let availableSize = size;
        for (const order of orderbook) {
            acc += order[1];
            if (acc >= size) {
                selectedOrder = order;
                break;
            }
        }
        if (!selectedOrder) {
            selectedOrder = orderbook[orderbook.length - 1];
            availableSize = acc;
        }
        if (side === 'buy')
            return [selectedOrder[0] * 1.05, availableSize];
        else
            return [selectedOrder[0] * 0.95, availableSize];
    }
    getTopAskPrice(asks) {
        const a = asks?.length > 0 && Number(asks[0][0]);
        return a || 0;
    }
    getlastPrice(fills) {
        const last = fills && fills.length > 0 && fills[0].price;
        if (last)
            return last;
        return 0;
    }
    getMiddlePointPrice(bids, asks) {
        const b = bids?.length > 0 && Number(bids[0][0]);
        const a = asks?.length > 0 && Number(asks[0][0]);
        if (b && a)
            return (b + a) / 2;
        return 0;
    }
    getContractsOnOrderbooks(bids, asks) {
        let contracts = 0;
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        for (const [_, size] of bids) {
            contracts += size;
        }
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        for (const [_, size] of asks) {
            contracts += size;
        }
        return contracts;
    }
    async getOrderBook() {
        const decodedBids = this.market.state.inner.bids;
        const decodedAsks = this.market.state.inner.asks;
        const bidsData = await this.connection.getAccountInfo(decodedBids);
        const asksData = await this.connection.getAccountInfo(decodedAsks);
        const bids = this.orderbookParser(bidsData.data, false);
        const asks = this.orderbookParser(asksData.data, true);
        return { bids, asks };
    }
    async getOrderBookFull() {
        const decodedBids = this.market.state.inner.bids;
        const decodedAsks = this.market.state.inner.asks;
        const bidsData = await this.connection.getAccountInfo(decodedBids);
        const asksData = await this.connection.getAccountInfo(decodedAsks);
        const bids = aaob_1.Slab.deserialize(bidsData.data, shared_1.CALLBACK_INFO_LEN);
        const asks = aaob_1.Slab.deserialize(asksData.data, shared_1.CALLBACK_INFO_LEN);
        return { bids, asks };
    }
    async loadFills() {
        const eventQueue = await this.loadEventQueue(this.market.state.inner.eventQueue);
        const events = [...Array(eventQueue.header.head.toNumber()).keys()]
            .map((e) => eventQueue.parseEvent(e))
            .filter((e) => e instanceof aaob_1.EventFill);
        return events.map((fill) => {
            return {
                price: (0, tokenAmount_1.splToUiAmount)((0, tokenAmount_1.priceLotsToNative)((0, aaob_1.getPriceFromKey)(fill.makerOrderId).ushrn(32), this.market.state.inner.baseMultiplier, this.market.state.inner.quoteMultiplier, this.market.state.inner.config.decimals), shared_1.QUOTE_TOKEN_DECIMALS),
                amount: (0, tokenAmount_1.splToUiAmount)((0, tokenAmount_1.sizeLotsToNative)(fill.baseSize, this.market.state.inner.baseMultiplier), this.market.state.inner.config.decimals)
            };
        });
    }
    async loadEventQueue(address) {
        const eventQueue = await aaob_1.EventQueue.load(this.client.connection, address, shared_1.CALLBACK_INFO_LEN);
        return eventQueue;
    }
}
exports.FuturesMarketViewer = FuturesMarketViewer;
//# sourceMappingURL=futuresMarket.js.map